import time
from machine import Pin, I2C, PWM
from ssd1306 import SSD1306_I2C
import framebuf
import network
import urequests as requests
import time
import ujson
import sys
import _thread

i2c = I2C(sda=Pin(21), scl=Pin(22))
oled = SSD1306_I2C(128, 64, i2c)
buzzer = PWM(Pin(19, Pin.OUT))
buzzer.init(freq=0, duty=0)
ADAFRUIT_IO_USERNAME = "dim_thitisak"
ADAFRUIT_IO_KEY = "aio_JYoJ08q7Yq98FBfbipzw3eafoDDO"

def connectWifi(ssid, password):
    wlan = network.WLAN(network.STA_IF)
    if not wlan.active():
        wlan.active(True)

    if not wlan.isconnected():
        wlan.connect(ssid, password)
        print("connecting...")
        timeout = 0
        while not wlan.isconnected() and timeout < 10:
            print(10 - timeout)
            timeout += 1
            time.sleep(1)
    
    if wlan.isconnected():
        print('connected')
        print(wlan.ifconfig())
    else:
        print('not connected')
        sys.exit()
        
def getADAFRUIT(FEED_KEY):
    # use adafruit to post and get data
    url = 'https://io.adafruit.com/api/v2/' + ADAFRUIT_IO_USERNAME + '/feeds/' + FEED_KEY + '/data'
    header = {'X-AIO-Key': ADAFRUIT_IO_KEY,
        'Content-Type': 'application/json'}
    res = requests.get(url, headers=header)
    res = res.json()[0]['value']
    return res
    
def postADAFRUIT(FEED_KEY, data):
    url = 'https://io.adafruit.com/api/v2/' + ADAFRUIT_IO_USERNAME + '/feeds/' + FEED_KEY + '/data'
    header = {'X-AIO-Key': ADAFRUIT_IO_KEY,
        'Content-Type': 'application/json'}
    d = ujson.dumps({"value": data})
    res = requests.post(url, headers=header, data=d)
    res.close()

def realTime():
    # using default address 0x3C
    alarmTime = "XX:XX"
    isAlarm = False
    soundAlarm = [600, 850, 420, 550, 700]
    i = 0
    duty = 20
    check = False
    bDay = time.localtime()[2]
    while True:    
        cTime = time.localtime()
        # cTime : 0 = Year, 1 = Month, 2 = Day, 3 = Hour, 4 = Minute
        # cTimeSTR : Hour:Min
        cTimeSTR = str(cTime[3]) + ":"
        if cTime[3] < 10:
            cTimeSTR = "0" + cTimeSTR
        if cTime[4] < 10:
            cTimeSTR += "0"
        cTimeSTR += str(cTime[4])
        
        # cDate : Day/Month/Year
        cDate = str(cTime[2]) + "/" + str(cTime[1]) + "/" + str(cTime[0])
        
        if bDay != cTime[2]:
            bDay = cTime[2]
            check = False
        
        if (alarmTime == cTimeSTR) and not check:
            isAlarm = True
            i = 0
            buzzer.init(freq=440, duty=16)
            postADAFRUIT("isalarm", "1")
            check = True
            
        if isAlarm:
            status = getADAFRUIT("isalarm")
            print(status)
            if status == "0":
                isAlarm = False
                buzzer.deinit()
            else:
                buzzer.init(freq=600, duty=duty)
                    
        print(cTime)
        if cTime[5] % 10 == 0 and not isAlarm:
            oled.fill(0)
            if cTime[5] != 0:
                alarmTime = getADAFRUIT("alarmclock")
        
        oled.text(cTimeSTR , 45, 7, 1)
        oled.text(cDate, 30, 17, 1)
        oled.text("Alarmclock", 28, 37, 1)
        oled.text("|" + alarmTime + "|", 37, 47, 1)
        oled.show()
        time.sleep(0.3)

# connect wifi
WIFI_SSID = 'DIMMOYeiei'
WIFI_PASSWORD = '0824503177'
connectWifi(WIFI_SSID, WIFI_PASSWORD)

realTime()