/* mbed Microcontroller Library
 * Copyright (c) 2019 ARM Limited
 * SPDX-License-Identifier: Apache-2.0
 */

#include "mbed.h"
#include "DebounceIn.h"
#include "stdio.h"
#include "stdlib.h"
#include "time.h"
#include "OLEDDisplay.h"
#include <cstring>

#define SDA PB_7
#define SCL PB_6

EventQueue queue[2];
Thread thr_queue[2];
Timeout timer;
DigitalOut led3(PB_3);
DebounceIn sw1(D3, PullUp);
// I2CSlave slave(D4,D5);
OLEDDisplay oled;

char buf[20];

bool playState = false;
bool taskRunning = false;

volatile int randomNumber;
volatile int finalAnswer;

#define REFRESH_RATE 17ms

#define GAUGE_SYMBOL '+'
#define SELECT_SYMBOL '^'
volatile int gaugeCursor[2] = {3, 4};
volatile int gaugeStatus = 12;
volatile int gaugeRemain;
char gauge[20];

volatile int choiceCursor[2] = {2, 0};
volatile int choice = 2;

bool searchEnd = false;
volatile int numCursor[2] = {1, 0};
volatile int boundary[3];
volatile int shift = 1204322;
int sum = 0;
int countWrong = 0;
int countCorrect = 0;

void displayNodeName() {
    oled.cursor(0, 4);
    oled.printf("Guest a number!");
}

void loadGauge(int x) {
    for (int i = 0; i < x; i++) {
        gauge[i] = GAUGE_SYMBOL;
    }
}

void clearGauge() {
    for (int i = 0; i < gaugeStatus; i++) {
        gauge[i] = ' ';
    }
}

void clearline(int x) {
    oled.cursor(x, 0);
    oled.printf("                     ");
}

bool evaluateResult() {
    bool isCorrect = false;

    switch (choice) {
    case 0:
        if (randomNumber >= boundary[0] && randomNumber <= boundary[1]) {
            isCorrect = true;
            countCorrect++;
            boundary[2] = boundary[1];
            boundary[1] = (boundary[2] + boundary[0])/2;

        } else {
            isCorrect = false;
            countWrong++;
        }
        break;

    case 1:
        if (randomNumber == boundary[1]) {
            isCorrect = true;
            countCorrect++;
            searchEnd = true;

        } else {
            isCorrect = false;
            countWrong++;
        }
        break;

    case 2:
        if (randomNumber >= boundary[1] + 1 && randomNumber <= boundary[2]) {
            isCorrect = true;
            countCorrect++;
            boundary[0] = boundary[1] + 1;
            boundary[1] = (boundary[2] + boundary[0])/2;

        } else {
            isCorrect = false;
            countWrong++;
        }
        break;

    }

    sum += (choice + 1);
    return isCorrect;
}

void numberFound() {
    clearline(numCursor[0]);
    clearline(choiceCursor[0]);
    oled.cursor(choiceCursor[0], 0);
    oled.printf("      %sFound! %d\n", randomNumber / 10 == 0 ? " " : " ", randomNumber);
    oled.printf("     Z = (30%9)+3");
    playState = false;
    led3 = 0;

    finalAnswer = ((randomNumber * sum + shift) % (countWrong * countCorrect * 6 + 6)) % 10;
    printf("Final Answer: %d\n", finalAnswer);

    buf[1] = finalAnswer;
}

void dislplaySelection() {
    if (searchEnd) {
        return;
    }

    taskRunning = true;

    clearline(numCursor[0]);

    while (gaugeRemain >= 0) {
        clearGauge();
        loadGauge(gaugeRemain);

        oled.cursor(gaugeCursor[0], gaugeCursor[1]);
        oled.printf("[%s]", gauge);
        gaugeRemain -= 2;

        oled.cursor(numCursor[0], numCursor[1]);
        oled.printf("%5d-%-d%s  %2d  %2d-%-d", boundary[0], boundary[1], boundary[1] / 10 == 0 ? " " : "", 
        boundary[1], boundary[1] + 1, boundary[2]);
    
        oled.cursor(choiceCursor[0], choiceCursor[1]);
        oled.printf("     %c     %c    %c", choice == 0 ? SELECT_SYMBOL : ' ', choice == 1 ? SELECT_SYMBOL : ' ', choice == 2 ? SELECT_SYMBOL : ' ');

        ThisThread::sleep_for(REFRESH_RATE);
    }

    bool res = evaluateResult();
    clearline(gaugeCursor[0]);

    if (searchEnd) {
       numberFound();

    } else {
        if (res) {
            oled.cursor(gaugeCursor[0], 0);
            oled.printf("          Yes");

        } else {

            oled.cursor(gaugeCursor[0], 0);
            oled.printf("       Nice try!");
        } 
    }

    choice = 2;
    taskRunning = false;
}

void sw1_pressed_defer(void) { 
    dislplaySelection();
}

void sw1_pressed(void) {
    if (playState) {
        gaugeRemain = gaugeStatus;
        choice++;

        if (choice == 3) {
            choice = 0;
        }

        if (!taskRunning) {
            queue[0].call(sw1_pressed_defer);
        }
    }
}

int start() {
    printf("Node 5 started...\n");

    taskRunning = false;
    playState = true;
    led3 = 1;
    oled.clear();
    
    finalAnswer = -1;
    randomNumber = 30;
    printf("Number: %d\n", randomNumber);
    boundary[0] = 0;
    boundary[1] = 50;
    boundary[2] = 99;
    searchEnd = false;
    choice = 2;
    
    displayNodeName();

    return randomNumber;
}

void stop() {
    playState = false;
    led3 = 0;
    finalAnswer = -1;
    oled.clear();
}

int main() {
    printf("Node 5 ready...\n");
    thr_queue[0].start(
        callback(&queue[0], &EventQueue::dispatch_forever));
    sw1.fall(sw1_pressed);
    // thr_queue[1].start(comm);
    
    start();
}